// hlavickove subory *******************************************************
// systemove funkcie
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
// GL a GLU kniznica
#include <GL/gl.h>
//#include <GL/glu.h>

// .h subor
#include "mPSystem.h"

#define PI    3.141592653
#define PI2   6.283185306
#define PI180 0.017453293

// konstruktor *************************************************************
mPSystem::mPSystem()
{
 FireworksNum=0;
	
 ActualTime=0.0f; Fps=0.0f; T1=0.0f;

 RM=(GLfloat) RAND_MAX;

 srandom(1000);
}

// destruktor **************************************************************
mPSystem::~mPSystem()
{

}

// ziskanie nahodneho cisla v intervale start-stop *************************
GLfloat mPSystem::get_random_number(GLfloat start,GLfloat stop)
{
 // ziskanie nahodneho cisla v intervale 0.0f-1.0f
 R=(GLfloat) random(); R/=RM;

 // nastavenie minimalnej a maximalnej hodnoty
 R=R*(stop-start)+start;

 return(R);
}

// nastavenie poctu aktivnych ohnostrojov **********************************
void mPSystem::SetFireworksNum(GLuint FN)
{
 if (FN<=10) FireworksNum=FN;
}

// nastavenie aktualneho casu **********************************************
void mPSystem::SetTime(GLfloat AT,GLfloat FPS)
{
 if (FPS!=0.0f) Fps=FPS;
 else Fps=1.0f;

 ActualTime+=AT/Fps*50.0f;

 // INFO: vypis aktualneho casu
 //fprintf(stderr,"%f\n",ActualTime);
}

 // pociatocne nastavenie parametrov ohnostroja ****************************
void mPSystem::FireworkSetup(GLuint N)
{
 // X,Y,Z suradnice polohy ohnostroja
 Firework[N].X=get_random_number(-40.0f,40.0f);
 Firework[N].Y=-60.0f;
 Firework[N].Z=get_random_number(-20.0f,0.0f);
 // zmena polohy v danom smere
 Firework[N].dX=get_random_number(-0.0005f,0.0005f);
 Firework[N].dY=0.006f;
 Firework[N].dZ=get_random_number(-0.0005f,0.0005f);

 // start "zivota" ohnostroja
 Firework[N].StartTime=ActualTime;
 // koniec "zivota" ohnostroja
 Firework[N].StopTime=get_random_number(16.0f,24.0f);

 // start "zivota" explozie
 Firework[N].ExplosionStart=get_random_number(10.0f,12.0f);
 // dlzka "zivota" explozie
 Firework[N].ExplosionTime=0.0f;
 // typ explozie
 Firework[N].ExplosionType=(GLuint) get_random_number(0.0f,3.0f);
 // INFO: start explozie a stop ohnostroja
 //fprintf(stderr,"%f %f\n",Firework[N].ExplosionStart,Firework[N].StopTime);

 // vyber farby explozie
 R1=get_random_number(0.0f,1.0f); // R
 R2=get_random_number(0.0f,1.0f); // G
 R3=get_random_number(0.0f,1.0f); // B

 // zvacsovanie sa polomeru explozie - podla typu
 switch (Firework[N].ExplosionType) {
  case 0: R=get_random_number(0.1f,2.0f); break;
  case 1: break;
  case 2: R=get_random_number(0.1f,2.0f); break;
  default: break;
 }

 // 64 castic pre exploziu
 for (j=0;j<64;j++) {
  // X,Y,Z suradnice polohy castic
  Firework[N].Expl[j].X=Firework[N].X;
  Firework[N].Expl[j].Y=Firework[N].Y;
  Firework[N].Expl[j].Z=Firework[N].Z;
  // zmena polohy v danom smere
  switch (Firework[N].ExplosionType) {
   // zvacsovanie polomeru je konstantne pre kazdu casticu - kruznica
   case 0: Firework[N].Expl[j].dX=R*cos((GLfloat) (20*j)*PI180);
           Firework[N].Expl[j].dY=R*sin((GLfloat) (20*j)*PI180);
           Firework[N].Expl[j].dZ=0.0f;
           break;
   // zvacsovanie polomeru sa meni pre kazdu casticu
   case 1: R=get_random_number(0.1f,2.0f);
           Firework[N].Expl[j].dX=R*cos((GLfloat) (20*j)*PI180);
           Firework[N].Expl[j].dY=R*sin((GLfloat) (20*j)*PI180);
           Firework[N].Expl[j].dZ=R*cos((GLfloat) (10*j)*PI180);
           break;
   // hviezda
   case 2: Firework[N].Expl[j].dX=((GLfloat) (j%4)/R)*cos((GLfloat) (20*j)*PI180);
           Firework[N].Expl[j].dY=((GLfloat) (j%4)/R)*sin((GLfloat) (20*j)*PI180);
           Firework[N].Expl[j].dZ=0.0f;
           break;
   default: break;
  }
  // farba castice
  Firework[N].Expl[j].R=R1;
  Firework[N].Expl[j].G=R2;
  Firework[N].Expl[j].B=R3;
 }
}

// vykreslenie vsetkych aktivnych ohnostrojov ******************************
void mPSystem::FireworksDraw(void)
{
 glPushMatrix();
 glScalef(1.5f,1.5f,1.5f);

 for (i=0;i<FireworksNum;i++) {
  T1=(ActualTime-Firework[i].StartTime)*50.0f;

  if (T1<Firework[i].ExplosionStart) { // ak nedoslo k explozii
   FireworkRocket(i); // vykreslenie strely

   fade=(T1-T1*T1/15.0f)+10.0f; // utlm pohybu ohnostroja pocas strely
   // vypocet novej polohy ohnostroja
   Firework[i].X+=Firework[i].dX*fade;
   Firework[i].Y+=Firework[i].dY*fade;
   Firework[i].Z+=Firework[i].dZ*fade;
  }
  else if (T1<Firework[i].StopTime) {  // doslo k explozii
        // nastavenie dlzky "zivota" explozie
        if (Firework[i].ExplosionTime==0.0f) Firework[i].ExplosionTime=(Firework[i].StopTime-T1);

        FireworkExplode(i); // vykreslenie explozie

        fade=(T1-T1*T1/10.0f)+5.0f; // utlm pohybu ohnostroja pocas explozie
        // vypocet novej polohy ohnostroja
        Firework[i].X+=Firework[i].dX*fade;
        Firework[i].Y+=Firework[i].dY*fade;
        Firework[i].Z+=Firework[i].dZ*fade;
       }
	   else FireworkSetup(i);          // ohnostroj "ukoncil svoj zivot", vytvori sa novy
 }

 glPopMatrix();
}

// vyvoj "zivota" strely ***************************************************
void mPSystem::FireworkRocket(GLuint N)
{
 // vykreslenie strely
 for (j=0;j<8;j++) {
  ii=(GLfloat) j;
  Firework[N].Rocket[j].X=Firework[N].X-Firework[N].dX*ii*16.0f;
  Firework[N].Rocket[j].Y=Firework[N].Y-Firework[N].dY*ii*16.0f;
  Firework[N].Rocket[j].Z=Firework[N].Z-Firework[N].dZ*ii*16.0f;

  glColor4f(1.0f,1.0f,1.0f,(8.0f-ii)/32.0f);
  glBegin(GL_QUADS);
   glTexCoord2f(0.0f,0.0f); glVertex3f(Firework[N].Rocket[j].X-1.0f,Firework[N].Rocket[j].Y-1.0f,Firework[N].Rocket[j].Z);
   glTexCoord2f(1.0f,0.0f); glVertex3f(Firework[N].Rocket[j].X+1.0f,Firework[N].Rocket[j].Y-1.0f,Firework[N].Rocket[j].Z);
   glTexCoord2f(1.0f,1.0f); glVertex3f(Firework[N].Rocket[j].X+1.0f,Firework[N].Rocket[j].Y+1.0f,Firework[N].Rocket[j].Z);
   glTexCoord2f(0.0f,1.0f); glVertex3f(Firework[N].Rocket[j].X-1.0f,Firework[N].Rocket[j].Y+1.0f,Firework[N].Rocket[j].Z);
  glEnd();
 }
}

// vyvoj "zivota" explozie *************************************************
void mPSystem::FireworkExplode(GLuint N)
{
 // utlm pohybu castic explozie
 fade=(T1-sqrt(T1))/3.0f;

 // alfa hodnota farby castic explozie
 R=((Firework[N].StopTime-T1)/Firework[N].ExplosionTime);

 for (j=0;j<64;j++) {
  Firework[N].Expl[j].X=Firework[N].X+Firework[N].Expl[j].dX*fade;
  Firework[N].Expl[j].Y=Firework[N].Y+Firework[N].Expl[j].dY*fade;
  Firework[N].Expl[j].Z=Firework[N].Z+Firework[N].Expl[j].dZ*fade;

  glColor4f(Firework[N].Expl[j].R,Firework[N].Expl[j].G,Firework[N].Expl[j].B,R);
  glBegin(GL_QUADS);
   glTexCoord2f(0.0f,0.0f); glVertex3f(Firework[N].Expl[j].X-1.0f,Firework[N].Expl[j].Y-1.0f,Firework[N].Expl[j].Z);
   glTexCoord2f(1.0f,0.0f); glVertex3f(Firework[N].Expl[j].X+1.0f,Firework[N].Expl[j].Y-1.0f,Firework[N].Expl[j].Z);
   glTexCoord2f(1.0f,1.0f); glVertex3f(Firework[N].Expl[j].X+1.0f,Firework[N].Expl[j].Y+1.0f,Firework[N].Expl[j].Z);
   glTexCoord2f(0.0f,1.0f); glVertex3f(Firework[N].Expl[j].X-1.0f,Firework[N].Expl[j].Y+1.0f,Firework[N].Expl[j].Z);
  glEnd();
 }
}
